MODULE mo_example

  USE error_handling
  IMPLICIT NONE

CONTAINS

  INTEGER FUNCTION division(number1, number2, ifail)
    INTEGER,       INTENT(IN)    :: number1, number2
    TYPE(t_error), INTENT(INOUT) :: ifail

    division = 0
    IF (number2 == 0) THEN
      CALL create_error(ifail, t_arithmetic_error("Division by zero!"))
    ELSE IF ((number1 < 0) .OR. (number2 < 0)) THEN
      CALL create_error(ifail, t_invalid_input_error("Only positive numbers allowed!"))
    ELSE
      division = number1 / number2
    END IF
  END FUNCTION division
END MODULE mo_example



PROGRAM example

  USE mo_example

  USE mtime,          ONLY: setCalendar, PROLEPTIC_GREGORIAN  
  USE mtime_hl,       ONLY: t_datetime, t_timedelta
  USE error_handling
  
  IMPLICIT NONE
  
  TYPE(t_datetime)  :: dt, dt2, dt3
  TYPE(t_timedelta) :: td

  INTEGER       :: test_number1, test_number2, test_result
  TYPE(t_error) :: ifail


  WRITE (0,*) "example_hl : test example"

  WRITE (0,*) "  setting calendar."
  CALL setCalendar(PROLEPTIC_GREGORIAN)

  ! Instead of:  TYPE(datetime), POINTER :: dt
  !              dt => newDatetime("1970-01-01T00:00:00")
  !                [...]
  !              CALL deallocateDatetime(dt)

  ! TYPE(t_datetime)  :: dt
  WRITE (0,*) "  testing assignment of t_datetime."
  dt = "1970-01-01T00:00:00"

  ! Instead of:  TYPE(timedelta), POINTER :: td
  !              td => newTimeDelta("PT1H1M1S")
  !                [...]
  !              CALL deallocateTimedelta(td)

  ! TYPE(t_timedelta) :: td
  WRITE (0,*) "  testing assignment of t_timedelta."
  td = "PT1H1M1S"

  ! Instead of:  CALL datetimeToString(dt, dstring1)
  !              CALL timedeltaToString(td, dstring2)
  !              WRITE (0,*) "compute ", dstring1, " + ", dstring2

  WRITE (0,*) "compute ", dt%to_string(), " + ", td%to_string() 

  ! Adding time intervals is unchanged:

  dt = dt + td
  WRITE (0,*) "result: ", dt%to_string()

  ! --- Further examples:

  ! subtraction of two dates
  dt2 = "1970-01-01T00:00:00"
  td = dt-dt2
  WRITE (0,*) "subtraction of dates: time delta: ", td%to_string()

  ! comparison of dates
  dt  = "1970-01-01T00:00:00"
  dt2 = "1970-01-01T00:00:00"
  WRITE (0,*) dt%to_string(), " == ", dt2%to_string(), ": ", (dt == dt2)
  dt3 = "1970-01-01T00:00:01"
  WRITE (0,*) dt%to_string(), " == ", dt3%to_string(), ": ", (dt == dt3)

  ! interval assignment with milliseconds
  td = 360000
  WRITE (0,*) "interval assignment with milliseconds: ", td%to_string()

  ! ------------------------------------------------------------
  ! EXAMPLE FOR ERROR HANDLING
  ! ------------------------------------------------------------

  WRITE (0,*) "EXAMPLE FOR ERROR HANDLING"
  test_number1 = 10
  test_number2 = -1
  test_result  = division(test_number1, test_number2, ifail)

  SELECT TYPE(info => ifail%info)
  TYPE is (t_arithmetic_error)
    WRITE (0,*) "I'm too stupid!"
    CALL ifail%report()
  TYPE is (t_invalid_input_error)
    WRITE (0,*) "You are too stupid!"
    CALL ifail%report()
  CLASS default
    WRITE (0,*) "Everything was ok!"
    WRITE (0,*) "result = ", test_result
  END SELECT
  
END PROGRAM example
